@@ -2,7 +2,7 @@
 from __future__ import print_function, division, absolute_import
 import sys, argparse, os
 from treetime.wrappers import ancestral_reconstruction, mugration, scan_homoplasies, timetree, estimate_clock_model
-import treetime
+from treetime import version
 
 py2 = sys.version_info.major==2
 
@@ -148,6 +148,7 @@ def add_gtr_arguments(parser):
 def add_anc_arguments(parser):
     parser.add_argument('--keep-overhangs', default = False, action='store_true', help='do not fill terminal gaps')
     parser.add_argument('--zero-based', default = False, action='store_true', help='zero based mutation indexing')
+    parser.add_argument('--reconstruct-tip-states', default = False, action='store_true', help='overwrite ambiguous states on tips with the most likely inferred state')
     parser.add_argument('--report-ambiguous', default=False, action="store_true", help='include transitions involving ambiguous states')
 
 
@@ -169,6 +170,8 @@ def make_parser():
     t_parser.add_argument('--tree', type=str, help=tree_description)
     add_seq_len_aln_group(t_parser)
     t_parser.add_argument('--dates', type=str, help=dates_description)
+    t_parser.add_argument('--name-column', type=str, help="label of the column to be used as taxon name")
+    t_parser.add_argument('--date-column', type=str, help="label of the column to be used as sampling date")
     add_reroot_group(t_parser)
     add_gtr_arguments(t_parser)
     t_parser.add_argument('--clock-rate', type=float, help="if specified, the rate of the molecular clock won't be optimized.")
@@ -189,6 +192,8 @@ def make_parser():
                         help='maximal number of iterations the inference cycle is run. Note that for polytomy resolution and coalescence models max_iter should be at least 2')
     t_parser.add_argument('--coalescent', default="0.0", type=str,
                           help=coalescent_description)
+    t_parser.add_argument('--n-skyline', default="20", type=int,
+                          help="number of grid points in skyline coalescent model")
     t_parser.add_argument('--plot-tree', default="timetree.pdf",
                             help = "filename to save the plot to. Suffix will determine format"
                                    " (choices pdf, png, svg, default=pdf)")
@@ -201,6 +206,7 @@ def make_parser():
                             help = "don't show tip labels (default for small trees with >=30 leaves)")
     add_anc_arguments(t_parser)
     add_common_args(t_parser)
+    t_parser.add_argument("--version", action="version", version="%(prog)s " + version)
 
     def toplevel(params):
         if (params.aln or params.tree) and params.dates:
@@ -229,9 +235,9 @@ def make_parser():
     ## ANCESTRAL RECONSTRUCTION
     a_parser = subparsers.add_parser('ancestral', description=ancestral_description)
     add_aln_group(a_parser)
-    a_parser.add_argument('--tree', type = str,  help =tree_description)
+    a_parser.add_argument('--tree', type=str,  help=tree_description)
     add_gtr_arguments(a_parser)
-    a_parser.add_argument('--marginal', default = False, action="store_true", help ="marginal reconstruction of ancestral sequences")
+    a_parser.add_argument('--marginal', default=False, action="store_true", help ="marginal reconstruction of ancestral sequences")
     add_anc_arguments(a_parser)
     add_common_args(a_parser)
     a_parser.set_defaults(func=ancestral_reconstruction)
@@ -239,6 +245,7 @@ def make_parser():
     ## MUGRATION
     m_parser = subparsers.add_parser('mugration', description=mugration_description)
     m_parser.add_argument('--tree', required = True, type=str, help=tree_description)
+    m_parser.add_argument('--name-column', type=str, help="label of the column to be used as taxon name")
     m_parser.add_argument('--attribute', type=str, help ="attribute to reconstruct, e.g. country")
     m_parser.add_argument('--states', required = True, type=str, help ="csv or tsv file with discrete characters."
                                     "\n#name,country,continent\ntaxon1,micronesia,oceania\n...")
@@ -265,6 +272,8 @@ def make_parser():
                         "signal and recalculate branch length unless run with --keep_root.")
     c_parser.add_argument('--tree', required=True, type=str,  help=tree_description)
     c_parser.add_argument('--dates', required=True, type=str, help=dates_description)
+    c_parser.add_argument('--date-column', type=str, help="label of the column to be used as sampling date")
+    c_parser.add_argument('--name-column', type=str, help="label of the column to be used as taxon name")
     add_seq_len_aln_group(c_parser)
 
     add_reroot_group(c_parser)
@@ -279,7 +288,7 @@ def make_parser():
 
     # make a version subcommand
     v_parser = subparsers.add_parser('version', description='print version')
-    v_parser.set_defaults(func=lambda x: print(treetime.version))
+    v_parser.set_defaults(func=lambda x: print("treetime "+version))
 
     ## call the relevant function and return
     if py2:
