@@ -1,8 +1,9 @@
 from __future__ import print_function, division, absolute_import
 import numpy as np
 from treetime import config as ttconf
+from treetime import MissingDataError
 from .treeanc import TreeAnc
-from .utils import numeric_date, DateConversion
+from .utils import numeric_date, DateConversion, datestring_from_numeric
 from .distribution import Distribution
 from .branch_len_interpolator import BranchLenInterpolator
 from .node_interpolator import NodeInterpolator
@@ -79,8 +80,7 @@ class ClockTree(TreeAnc):
         self.clock_model=None
         self.use_covariation=use_covariation # if false, covariation will be ignored in rate estimates.
         self._set_precision(precision)
-        if self._assign_dates()==ttconf.ERROR:
-            raise ValueError("ClockTree requires date constraints!")
+        self._assign_dates()
 
 
     def _assign_dates(self):
@@ -92,8 +92,7 @@ class ClockTree(TreeAnc):
             success/error code
         """
         if self.tree is None:
-            self.logger("ClockTree._assign_dates: tree is not set, can't assign dates", 0)
-            return ttconf.ERROR
+            raise MissingDataError("ClockTree._assign_dates: tree is not set, can't assign dates")
 
         bad_branch_counter = 0
         for node in self.tree.find_clades(order='postorder'):
@@ -128,9 +127,9 @@ class ClockTree(TreeAnc):
                 bad_branch_counter += 1
 
         if bad_branch_counter>self.tree.count_terminals()-3:
-            self.logger("ERROR: ALMOST NO VALID DATE CONSTRAINTS, EXITING", 1, warn=True)
-            return ttconf.ERROR
+            raise MissingDataError("ERROR: ALMOST NO VALID DATE CONSTRAINTS")
 
+        self.logger("ClockTree._assign_dates: assigned date contraints to {} out of {} tips.".format(self.tree.count_terminals()-bad_branch_counter, self.tree.count_terminals()), 1)
         return ttconf.SUCCESS
 
 
@@ -149,7 +148,7 @@ class ClockTree(TreeAnc):
             self.precision=precision
             if self.one_mutation and self.one_mutation<1e-4 and precision<2:
                 self.logger("ClockTree._set_precision: FOR LONG SEQUENCES (>1e4) precision>=2 IS RECOMMENDED."
-                            " \n\t **** precision %d was specified by the user"%precision, level=0)
+                            " precision %d was specified by the user"%precision, level=0)
         else:
             # otherwise adjust it depending on the minimal sensible branch length
             if self.one_mutation:
@@ -263,7 +262,7 @@ class ClockTree(TreeAnc):
         """
         self.logger("ClockTree.init_date_constraints...",2)
         self.tree.coalescent_joint_LH = 0
-        if self.aln and (ancestral_inference or (not hasattr(self.tree.root, 'sequence'))):
+        if self.aln and (not self.sequence_reconstruction):
             self.infer_ancestral_sequences('probabilistic', marginal=self.branch_length_mode=='marginal',
                                             sample_from_profile='root',**kwarks)
 
@@ -286,9 +285,11 @@ class ClockTree(TreeAnc):
 
                 if self.branch_length_mode=='marginal':
                     node.profile_pair = self.marginal_branch_profile(node)
+                elif self.branch_length_mode=='joint' and (not hasattr(node, 'branch_state')):
+                    self.add_branch_state(node)
 
                 node.branch_length_interpolator = BranchLenInterpolator(node, self.gtr,
-                            pattern_multiplicity = self.multiplicity, min_width=self.min_width,
+                            pattern_multiplicity = self.data.multiplicity, min_width=self.min_width,
                             one_mutation=self.one_mutation, branch_length_mode=self.branch_length_mode)
 
                 node.branch_length_interpolator.merger_cost = merger_cost
@@ -312,8 +313,8 @@ class ClockTree(TreeAnc):
 
                 if hasattr(node, 'bad_branch') and node.bad_branch is True:
                     self.logger("ClockTree.init_date_constraints -- WARNING: Branch is marked as bad"
-                                ", excluding it from the optimization process.\n"
-                                "\t\tDate constraint will be ignored!", 4, warn=True)
+                                ", excluding it from the optimization process."
+                                " Date constraint will be ignored!", 4, warn=True)
             else: # node without sampling date set
                 node.raw_date_constraint = None
                 node.date_constraint = None
@@ -438,13 +439,14 @@ class ClockTree(TreeAnc):
 
             if node.joint_pos_Cx is None: # no constraints or branch is bad - reconstruct from the branch len interpolator
                 node.branch_length = node.branch_length_interpolator.peak_pos
-
+            elif node.date_constraint is not None and node.date_constraint.is_delta:
+                node.branch_length = node.up.time_before_present - node.date_constraint.peak_pos
             elif isinstance(node.joint_pos_Cx, Distribution):
                 # NOTE the Lx distribution is the likelihood, given the position of the parent
                 # (Lx.x = parent position, Lx.y = LH of the node_pos given Lx.x,
                 # the length of the branch corresponding to the most likely
                 # subtree is node.Cx(node.time_before_present))
-                subtree_LH = node.joint_pos_Lx(node.up.time_before_present)
+                # subtree_LH = node.joint_pos_Lx(node.up.time_before_present)
                 node.branch_length = node.joint_pos_Cx(max(node.joint_pos_Cx.xmin,
                                             node.up.time_before_present)+ttconf.TINY_NUMBER)
 
@@ -475,7 +477,7 @@ class ClockTree(TreeAnc):
 
         # add the root sequence LH and return
         if self.aln:
-            LH += self.gtr.sequence_logLH(self.tree.root.cseq, pattern_multiplicity=self.multiplicity)
+            LH += self.gtr.sequence_logLH(self.tree.root.cseq, pattern_multiplicity=self.data.multiplicity)
         return LH
 
 
@@ -525,7 +527,7 @@ class ClockTree(TreeAnc):
                 # no information
                 node.marginal_pos_Lx = None
             else: # all other nodes
-                if node.date_constraint is not None and node.date_constraint.is_delta: # there is a time constraint
+                if node.date_constraint is not None and node.date_constraint.is_delta: # there is a hard time constraint
                     # initialize the Lx for nodes with precise date constraint:
                     # subtree probability given the position of the parent node
                     # position of the parent node is given by the branch length
@@ -575,6 +577,8 @@ class ClockTree(TreeAnc):
             if node.up is None:
                 node.msg_from_parent = None # nothing beyond the root
             # all other cases (All internal nodes + unconstrained terminals)
+            elif node.date_constraint is not None and node.date_constraint.is_delta:
+                node.marginal_pos_LH = node.date_constraint
             else:
                 parent = node.up
                 # messages from the complementary subtree (iterate over all sister nodes)
@@ -584,8 +588,6 @@ class ClockTree(TreeAnc):
                 # if parent itself got smth from the root node, include it
                 if parent.msg_from_parent is not None:
                     complementary_msgs.append(parent.msg_from_parent)
-                elif parent.marginal_pos_Lx is not None:
-                    complementary_msgs.append(parent.marginal_pos_LH)
 
                 if len(complementary_msgs):
                     msg_parent_to_node = NodeInterpolator.multiply(complementary_msgs)
@@ -677,17 +679,7 @@ class ClockTree(TreeAnc):
                         "later than present day",4 , warn=True)
 
             node.numdate = now - years_bp
-
-            # set the human-readable date
-            year = np.floor(node.numdate)
-            days = max(0,365.25 * (node.numdate - year)-1)
-            try:  # datetime will only operate on dates after 1900
-                n_date = datetime(year, 1, 1) + timedelta(days=days)
-                node.date = datetime.strftime(n_date, "%Y-%m-%d")
-            except:
-                # this is the approximation not accounting for gap years etc
-                n_date = datetime(1900, 1, 1) + timedelta(days=days)
-                node.date = "%04d-%02d-%02d"%(year, n_date.month, n_date.day)
+            node.date = datestring_from_numeric(node.numdate)
 
 
     def branch_length_to_years(self):
@@ -722,8 +714,8 @@ class ClockTree(TreeAnc):
         params = params or {}
         if rate_std is None:
             if not (self.clock_model['valid_confidence'] and 'cov' in self.clock_model):
-                self.logger("ClockTree.calc_rate_susceptibility: need valid standard deviation of the clock rate to estimate dating error.", 1, warn=True)
-                return ttconf.ERROR
+                raise ValueError("ClockTree.calc_rate_susceptibility: need valid standard deviation of the clock rate to estimate dating error.")
+
             rate_std = np.sqrt(self.clock_model['cov'][0,0])
 
         current_rate = np.abs(self.clock_model['slope'])
