@@ -30,8 +30,7 @@ class GTR(object):
             of observing characters in the alphabet. This is used to
             implement ambiguous characters like 'N'=[1,1,1,1] which are
             equally likely to be any of the 4 nucleotides. Standard profile_maps
-            are defined in file seq_utils.py. If None is provided, no ambigous
-            characters are supported.
+            are defined in file seq_utils.py.
 
          logger : callable
             Custom logging function that should take arguments (msg, level, warn=False),
@@ -39,6 +38,7 @@ class GTR(object):
 
         """
         self.debug=False
+        self.is_site_specific=False
         if isinstance(alphabet, str):
             if alphabet not in alphabet_synonyms:
                 raise AttributeError("Unknown alphabet type specified")
@@ -48,13 +48,14 @@ class GTR(object):
                 self.profile_map = profile_maps[tmp_alphabet]
         else:
             # not a predefined alphabet
-            self.alphabet = alphabet
+            self.alphabet = np.array(alphabet)
             if prof_map is None: # generate trivial unambiguous profile map is none is given
                 self.profile_map = {s:x for s,x in zip(self.alphabet, np.eye(len(self.alphabet)))}
             else:
-                self.profile_map = prof_map
-
+                self.profile_map = {x if type(x) is str else x:k for x,k in prof_map.items()}
 
+        self.state_index={s:si for si,s in enumerate(self.alphabet)}
+        self.state_index.update({s:si for si,s in enumerate(self.alphabet)})
         if logger is None:
             def logger_default(*args,**kwargs):
                 """standard logging function if none provided"""
@@ -69,13 +70,6 @@ class GTR(object):
         self.n_states = len(self.alphabet)
         self.assign_gap_and_ambiguous()
 
-        # NEEDED TO BREAK RATE MATRIX DEGENERACY AND FORCE NP TO RETURN REAL ORTHONORMAL EIGENVECTORS
-        # ugly hack, but works and shouldn't affect results
-        tmp_rng_state = np.random.get_state()
-        np.random.seed(12345)
-        self.break_degen = np.random.random(size=(self.n_states, self.n_states))*1e-6
-        np.random.set_state(tmp_rng_state)
-
         # init all matrices with dummy values
         self.logger("GTR: init with dummy values!", 3)
         self.v = None # right eigenvectors
@@ -86,7 +80,7 @@ class GTR(object):
 
     def assign_gap_and_ambiguous(self):
         n_states = len(self.alphabet)
-        self.logger("GTR: with alphabet: "+str(self.alphabet),1)
+        self.logger("GTR: with alphabet: "+str([x for x in self.alphabet]),1)
         # determine if a character exists that corresponds to no info, i.e. all one profile
         if any([x.sum()==n_states for x in self.profile_map.values()]):
             amb_states = [c for c,x in self.profile_map.items() if x.sum()==n_states]
@@ -97,7 +91,7 @@ class GTR(object):
 
         # check for a gap symbol
         try:
-            self.gap_index = list(self.alphabet).index('-')
+            self.gap_index = self.state_index['-']
         except:
             self.logger("GTR: no gap symbol!", 4, warn=True)
             self.gap_index=None
@@ -134,7 +128,10 @@ class GTR(object):
            and the equilibrium frequencies to obtain the rate matrix
            of the GTR model
         """
-        return (self.W*self.Pi).T
+        Q_tmp = (self.W*self.Pi).T
+        Q_diag = -np.sum(Q_tmp, axis=0)
+        np.fill_diagonal(Q_tmp, Q_diag)
+        return Q_tmp
 
 
 ######################################################################
@@ -155,18 +152,18 @@ class GTR(object):
         if not multi_site:
             eq_freq_str += "\nEquilibrium frequencies (pi_i):\n"
             for a,p in zip(self.alphabet, self.Pi):
-                eq_freq_str+='  '+str(a)+': '+str(np.round(p,4))+'\n'
+                eq_freq_str+='  '+a+': '+str(np.round(p,4))+'\n'
 
         W_str = "\nSymmetrized rates from j->i (W_ij):\n"
-        W_str+='\t'+'\t'.join(map(str, self.alphabet))+'\n'
+        W_str+='\t'+'\t'.join(self.alphabet)+'\n'
         for a,Wi in zip(self.alphabet, self.W):
-            W_str+= '  '+str(a)+'\t'+'\t'.join([str(np.round(max(0,p),4)) for p in Wi])+'\n'
+            W_str+= '  '+a+'\t'+'\t'.join([str(np.round(max(0,p),4)) for p in Wi])+'\n'
 
         if not multi_site:
             Q_str = "\nActual rates from j->i (Q_ij):\n"
-            Q_str+='\t'+'\t'.join(map(str, self.alphabet))+'\n'
+            Q_str+='\t'+'\t'.join(self.alphabet)+'\n'
             for a,Qi in zip(self.alphabet, self.Q):
-                Q_str+= '  '+str(a)+'\t'+'\t'.join([str(np.round(max(0,p),4)) for p in Qi])+'\n'
+                Q_str+= '  '+a+'\t'+'\t'.join([str(np.round(max(0,p),4)) for p in Qi])+'\n'
 
         return eq_freq_str + W_str + Q_str
 
@@ -190,6 +187,7 @@ class GTR(object):
         """
         n = len(self.alphabet)
         self._mu = mu
+        self.is_site_specific=False
 
         if pi is not None and len(pi)==n:
             Pi = np.array(pi)
@@ -213,7 +211,11 @@ class GTR(object):
             W=np.array(W)
 
         self._W = 0.5*(W+W.T)
-        self._check_fix_Q(fixed_mu=True)
+        np.fill_diagonal(W,0)
+        average_rate = W.dot(self.Pi).dot(self.Pi)
+        self._W = W/average_rate
+        self._mu *=average_rate
+
         self._eig()
 
 
@@ -508,8 +510,8 @@ class GTR(object):
         if gtr.gap_index is not None:
             if pi[gtr.gap_index]<gap_limit:
                 gtr.logger('The model allows for gaps which are estimated to occur at a low fraction of %1.3e'%pi[gtr.gap_index]+
-                       '\n\t\tthis can potentially result in artificats.'+
-                       '\n\t\tgap fraction will be set to %1.4f'%gap_limit,2,warn=True)
+                       ' this can potentially result in artificats.'+
+                       ' gap fraction will be set to %1.4f'%gap_limit,2,warn=True)
             pi[gtr.gap_index] = gap_limit
             pi /= pi.sum()
 
@@ -519,39 +521,13 @@ class GTR(object):
 ########################################################################
 ### prepare model
 ########################################################################
-    def _check_fix_Q(self, fixed_mu=False):
-        """
-        Check the main diagonal of Q and fix it in case it does not corresond
-        the definition of the rate matrix. Should be run every time when creating
-        custom GTR model.
-        """
-
-        # NEEDED TO BREAK RATE MATRIX DEGENERACY AND FORCE NP TO RETURN REAL ORTHONORMAL EIGENVECTORS
-        self._W += self.break_degen + self.break_degen.T
-        # fix W
-        np.fill_diagonal(self.W, 0)
-        Wdiag = -(self.Q).sum(axis=0)/self.Pi
-        np.fill_diagonal(self.W, Wdiag)
-        scale_factor = -np.sum(np.diagonal(self.Q)*self.Pi)
-        self._W /= scale_factor
-        if not fixed_mu:
-            self._mu *= scale_factor
-        if (self.Q.sum(axis=0) < 1e-10).sum() <  self.alphabet.shape[0]: # fix failed
-            print ("Cannot fix the diagonal of the GTR rate matrix. Should be all zero", self.Q.sum(axis=0))
-            import ipdb; ipdb.set_trace()
-            raise ArithmeticError("Cannot fix the diagonal of the GTR rate matrix.")
-
-
     def _eig(self):
         """
         Perform eigendecompositon of the rate matrix and stores the left- and right-
         matrices to convert the sequence profiles to the GTR matrix eigenspace
         and hence to speed-up the computations.
         """
-        W_nodiag = np.copy(self.W)
-        np.fill_diagonal(W_nodiag, 0)
-
-        self.eigenvals, self.v, self.v_inv = self._eig_single_site(W_nodiag, self.Pi)
+        self.eigenvals, self.v, self.v_inv = self._eig_single_site(self.W, self.Pi)
 
 
     def _eig_single_site(self, W, p):
@@ -574,7 +550,7 @@ class GTR(object):
         return eigvals, tmp_v.T/one_norm, (eigvecs*one_norm).T/tmpp
 
 
-    def compress_sequence_pair(self, seq_p, seq_ch, pattern_multiplicity=None,
+    def state_pair(self, seq_p, seq_ch, pattern_multiplicity=None,
                                ignore_gaps=False):
         '''
         Make a compressed representation of a pair of sequences, only counting
@@ -615,7 +591,7 @@ class GTR(object):
 
         from collections import Counter
         if seq_ch.shape != seq_p.shape:
-            raise ValueError("GTR.compress_sequence_pair: Sequence lengths do not match!")
+            raise ValueError("GTR.state_pair: Sequence lengths do not match!")
 
         if len(self.alphabet)<10: # for small alphabet, repeatedly check array for all state pairs
             pair_count = []
@@ -724,7 +700,7 @@ class GTR(object):
             Resulting probability
 
         """
-        seq_pair, multiplicity = self.compress_sequence_pair(seq_p, seq_ch,
+        seq_pair, multiplicity = self.state_pair(seq_p, seq_ch,
                                         pattern_multiplicity=pattern_multiplicity, ignore_gaps=ignore_gaps)
         return self.prob_t_compressed(seq_pair, multiplicity, t, return_log=return_log)
 
@@ -752,20 +728,21 @@ class GTR(object):
             If True, ignore gaps in distance calculations
 
         '''
-        seq_pair, multiplicity = self.compress_sequence_pair(seq_p, seq_ch,
-                                                            pattern_multiplicity = pattern_multiplicity,
-                                                            ignore_gaps=ignore_gaps)
+        seq_pair, multiplicity = self.state_pair(seq_p, seq_ch,
+                                        pattern_multiplicity = pattern_multiplicity,
+                                        ignore_gaps=ignore_gaps)
         return self.optimal_t_compressed(seq_pair, multiplicity)
 
 
     def optimal_t_compressed(self, seq_pair, multiplicity, profiles=False, tol=1e-10):
         """
-        Find the optimal distance between the two sequences, for compressed sequences
+        Find the optimal distance between the two sequences represented as state_pairs
+        or as pair of profiles
 
         Parameters
         ----------
 
-         seq_pair : compressed_sequence_pair
+         seq_pair : state_pair, tuple
             Compressed representation of sequences along a branch, either
             as tuple of state pairs or as tuple of profiles.
 
@@ -779,7 +756,7 @@ class GTR(object):
             either end of the branch. With profiles==True, optimization is performed
             while summing over all possible states of the nodes at either end of the
             branch. Note that the meaning/format of seq_pair and multiplicity
-            depend on the value of profiles.
+            depend on the value of :profiles:.
 
         """
 
@@ -1032,14 +1009,6 @@ class GTR(object):
                             char_dist=self.Pi,
                             flow_matrix=self.W)
 
-    def save_to_json(self, zip):
-        d = {
-        "full_gtr": self.mu * np.dot(self.Pi, self.W),
-        "Substitution rate" : self.mu,
-        "Equilibrium character composition": self.Pi,
-        "Flow rate matrix": self.W
-        }
-
 
 if __name__ == "__main__":
     pass
