@@ -25,6 +25,7 @@ class GTR_site_specific(GTR):
         self.seq_len=seq_len
         self.approximate = approximate
         super(GTR_site_specific, self).__init__(**kwargs)
+        self.is_site_specific=True
 
 
     @property
@@ -57,25 +58,34 @@ class GTR_site_specific(GTR):
             Equilibrium frequencies
 
         """
+        if not np.isscalar(mu) and pi is not None and len(pi.shape)==2:
+            if mu.shape[0]!=pi.shape[1]:
+                raise ValueError("GTR_site_specific: length of rate vector (got {}) and equilibrium frequency vector (got {}) must match!".format(mu.shape[0], pi.shape[1]))
+
         n = len(self.alphabet)
         if np.isscalar(mu):
             self._mu = mu*np.ones(self.seq_len)
         else:
             self._mu = np.copy(mu)
+            self.seq_len = mu.shape[0]
 
-        if pi is not None and pi.shape[0]==n:
-            self.seq_len = pi.shape[-1]
+        if pi is not None and pi.shape[0]==n and len(pi.shape)==2:
+            self.seq_len = pi.shape[1]
             Pi = np.copy(pi)
         else:
-            if pi is not None and len(pi)!=n:
-                raise ArgumentError("GTR_site_specific: length of equilibrium frequency vector does not match alphabet length.")
-            Pi = np.ones(shape=(n,self.seq_len))
+            if pi is not None:
+                if len(pi)==n:
+                    Pi = np.repeat([pi], self.seq_len, axis=0).T
+                else:
+                    raise ValueError("GTR_site_specific: length of equilibrium frequency vector (got {}) does not match alphabet length {}".format(len(pi), n))
+            else:
+                Pi = np.ones(shape=(n,self.seq_len))
 
         self._Pi = Pi/np.sum(Pi, axis=0)
 
         if W is None or W.shape!=(n,n):
             if (W is not None) and W.shape!=(n,n):
-                raise ArgumentError("GTR_site_specific: Size of substitution matrix does not match alphabet length.")
+                raise ValueError("GTR_site_specific: Size of substitution matrix (got {}) does not match alphabet length {}".format(W.shape, n))
             W = np.ones((n,n))
             np.fill_diagonal(W, 0.0)
             np.fill_diagonal(W, - W.sum(axis=0))
@@ -83,11 +93,13 @@ class GTR_site_specific(GTR):
             W=0.5*(np.copy(W)+np.copy(W).T)
 
         np.fill_diagonal(W,0)
-        avg_pi = self.Pi.mean(axis=-1)
-        average_rate = W.dot(avg_pi).dot(avg_pi)
+        average_rate = np.einsum('ia,ij,ja',self.Pi, W, self.Pi)/self.seq_len
+        # average_rate = W.dot(avg_pi).dot(avg_pi)
         self._W = W/average_rate
         self._mu *=average_rate
 
+
+        self.is_site_specific=True
         self._eig()
         self._make_expQt_interpolator()
 
@@ -124,6 +136,7 @@ class GTR_site_specific(GTR):
         gtr = cls(alphabet=alphabet, seq_len=L)
         n = gtr.alphabet.shape[0]
 
+        # Dirichlet distribution == l_1 normalized vector of samples of the Gamma distribution
         if pi_dirichlet_alpha:
             pi = 1.0*gamma.rvs(pi_dirichlet_alpha, size=(n,L))
         else:
@@ -143,7 +156,7 @@ class GTR_site_specific(GTR):
             mu = np.ones(L)
 
         gtr.assign_rates(mu=mu, pi=pi, W=W)
-        gtr.mu *= avg_mu/np.mean(gtr.mu)
+        gtr.mu *= avg_mu/np.mean(gtr.average_rate())
 
         return gtr
 
@@ -166,7 +179,7 @@ class GTR_site_specific(GTR):
             Equilibrium frequencies
 
          **kwargs:
-            Key word arguments to be passed
+            Key word arguments to be passed to the constructor
 
         Keyword Args
         ------------
@@ -255,15 +268,15 @@ class GTR_site_specific(GTR):
             p_ia_old = np.copy(p_ia)
             S_ij = np.einsum('a,ia,ja',mu_a, p_ia, T_ia)
             W_ij = (n_ij + n_ij.T + pc)/(S_ij + S_ij.T + pc)
-            
+
             avg_pi = p_ia.mean(axis=-1)
-            average_rate = W_ij.dot(avg_pi).dot(avg_pi)
+            average_rate = W_ij.dot(avg_pi).dot(avg_pi)  # crude approx, will be fixed in assign rates
             W_ij = W_ij/average_rate
             mu_a *=average_rate
-            
+
             p_ia = m_ia/(mu_a*np.dot(W_ij,T_ia)+Lambda)
             p_ia = p_ia/p_ia.sum(axis=0)
-            
+
             mu_a = n_a/(pc+np.einsum('ia,ij,ja->a', p_ia, W_ij, T_ia))
 
 
@@ -276,7 +289,7 @@ class GTR_site_specific(GTR):
                 if p_ia[gtr.gap_index,p]<gap_limit:
                     gtr.logger('The model allows for gaps which are estimated to occur at a low fraction of %1.3e'%p_ia[gtr.gap_index,p]+
                            '\n\t\tthis can potentially result in artifacts.'+
-                           '\n\t\tgap fraction will be set to %1.4f'%gap_limit,2,warn=True)
+                           '\n\t\tgap fraction will be set to %1.4f'%gap_limit,4,warn=True)
                 p_ia[gtr.gap_index,p] = gap_limit
                 p_ia[:,p] /= p_ia[:,p].sum()
 
@@ -456,7 +469,7 @@ class GTR_site_specific(GTR):
             logQt[np.isnan(logQt) | np.isinf(logQt) | bad_indices] = -ttconf.BIG_NUMBER
             seq_indices_c = np.zeros(len(seq_ch), dtype=int)
             seq_indices_p = np.zeros(len(seq_p), dtype=int)
-            for ai, a in self.alphabet:
+            for ai, a in enumerate(self.alphabet):
                 seq_indices_p[seq_p==a] = ai
                 seq_indices_c[seq_ch==a] = ai
 
