@@ -21,6 +21,9 @@ def base_regression(Q, slope=None):
     TYPE
         Description
     """
+    if np.isinf(Q).sum() or np.isnan(Q).sum():
+        raise ValueError("Invalid values in input data!")
+
     if slope is None:
         if (Q[tsqii] - Q[tavgii]**2/Q[sii])>0:
             slope = (Q[dtavgii] - Q[tavgii]*Q[davgii]/Q[sii]) \
@@ -355,7 +358,8 @@ class TreeRegression(object):
             bv = self.branch_value(n)
             var = self.branch_variance(n)
             for dx in [-0.001, 0.001]:
-                y = min(1.0, max(0.0, best_root["split"]+dx))
+                # y needs to be bounded away from 0 and 1 to avoid division by 0
+                y = min(0.9999, max(0.0001, best_root["split"]+dx))
                 tmpQ = self.propagate_averages(n, tv, bv*y, var*y) \
                      + self.propagate_averages(n, tv, bv*(1-y), var*(1-y), outgroup=True)
                 reg = base_regression(tmpQ, slope=slope)
@@ -381,6 +385,10 @@ class TreeRegression(object):
                  + self.propagate_averages(n, tv, bv*(1-x), var*(1-x), outgroup=True)
             return base_regression(tmpQ, slope=slope)['chisq']
 
+        if n.bad_branch or (n!=self.tree.root and n.up.bad_branch):
+            return np.nan, np.inf
+
+
         chisq_prox = np.inf if n.is_terminal() else base_regression(n.Qtot, slope=slope)['chisq']
         chisq_dist = np.inf if n==self.tree.root else base_regression(n.up.Qtot, slope=slope)['chisq']
 
@@ -423,6 +431,8 @@ class TreeRegression(object):
             regression parameters
         """
         best_root = self.find_best_root(force_positive=force_positive, slope=slope)
+        if best_root is None:
+            raise ValueError("Rerooting failed!")
         best_node = best_root["node"]
 
         x = best_root["split"]
