@@ -3,6 +3,7 @@ import numpy as np
 from scipy import optimize as sciopt
 from Bio import Phylo
 from treetime import config as ttconf
+from treetime import MissingDataError,UnknownMethodError,NotReadyError
 from .utils import tree_layout
 from .clock_tree import ClockTree
 
@@ -105,7 +106,8 @@ class TreeTime(ClockTree):
 
         use_covariation : bool, optional
             default False, if False, rate estimates will be performed using simple
-            regression ignoring phylogenetic covaration between nodes.
+            regression ignoring phylogenetic covaration between nodes. If vary_rate is True,
+            use_covariation is true by default
 
         **kwargs
            Keyword arguments needed by the downstream functions
@@ -120,11 +122,10 @@ class TreeTime(ClockTree):
         """
 
         # register the specified covaration mode
-        self.use_covariation = use_covariation
+        self.use_covariation = use_covariation or (vary_rate and (not type(vary_rate)==float))
 
-        if (self.tree is None) or (self.aln is None and self.seq_len is None):
-            self.logger("TreeTime.run: ERROR, alignment or tree are missing", 0)
-            return ttconf.ERROR
+        if (self.tree is None) or (self.aln is None and self.data.full_length is None):
+            raise MissingDataError("TreeTime.run: ERROR, alignment or tree are missing")
         if (self.aln is None):
             branch_length_mode='input'
 
@@ -132,7 +133,8 @@ class TreeTime(ClockTree):
 
         # determine how to reconstruct and sample sequences
         seq_kwargs = {"marginal_sequences":sequence_marginal or (self.branch_length_mode=='marginal'),
-                      "sample_from_profile":"root"}
+                      "sample_from_profile":"root",
+                      "reconstruct_tip_states":kwargs.get("reconstruct_tip_states", False)}
 
         tt_kwargs = {'clock_rate':fixed_clock_rate, 'time_marginal':False}
         tt_kwargs.update(kwargs)
@@ -160,11 +162,9 @@ class TreeTime(ClockTree):
             else:
                 plot_rtt=False
             reroot_mechanism = 'least-squares' if root=='clock_filter' else root
-            if self.clock_filter(reroot=reroot_mechanism, n_iqd=n_iqd, plot=plot_rtt, fixed_clock_rate=fixed_clock_rate)==ttconf.ERROR:
-                return ttconf.ERROR
+            self.clock_filter(reroot=reroot_mechanism, n_iqd=n_iqd, plot=plot_rtt, fixed_clock_rate=fixed_clock_rate)
         elif root is not None:
-            if self.reroot(root=root, clock_rate=fixed_clock_rate)==ttconf.ERROR:
-                return ttconf.ERROR
+            self.reroot(root=root, clock_rate=fixed_clock_rate)
 
         if self.branch_length_mode=='input':
             if self.aln:
@@ -181,8 +181,9 @@ class TreeTime(ClockTree):
         self.LH =[[seq_LH, self.tree.positional_joint_LH, 0]]
 
         if root is not None and max_iter:
-            if self.reroot(root='least-squares' if root=='clock_filter' else root, clock_rate=fixed_clock_rate)==ttconf.ERROR:
-                return ttconf.ERROR
+            new_root = self.reroot(root='least-squares' if root=='clock_filter' else root, clock_rate=fixed_clock_rate)
+            self.logger("###TreeTime.run: rerunning timetree after rerooting",0)
+            self.make_time_tree(**tt_kwargs)
 
         # iteratively reconstruct ancestral sequences and re-infer
         # time tree to ensure convergence.
@@ -241,14 +242,11 @@ class TreeTime(ClockTree):
         # rerun the estimation for variations of the rate
         if vary_rate:
             if type(vary_rate)==float:
-                res = self.calc_rate_susceptibility(rate_std=vary_rate, params=tt_kwargs)
+                self.calc_rate_susceptibility(rate_std=vary_rate, params=tt_kwargs)
             elif self.clock_model['valid_confidence']:
-                res = self.calc_rate_susceptibility(params=tt_kwargs)
+                self.calc_rate_susceptibility(params=tt_kwargs)
             else:
-                res = ttconf.ERROR
-
-            if res==ttconf.ERROR:
-                self.logger("TreeTime.run: rate variation failed and can't be used for confidence estimation", 1, warn=True)
+                raise UnknownMethodError("TreeTime.run: rate variation for confidence estimation is not available. Either specify it explicitly, or estimate from root-to-tip regression.")
 
         # if marginal reconstruction requested, make one more round with marginal=True
         # this will set marginal_pos_LH, which to be used as error bar estimations
@@ -325,8 +323,7 @@ class TreeTime(ClockTree):
 
         terminals = self.tree.get_terminals()
         if reroot:
-            if self.reroot(root='least-squares' if reroot=='best' else reroot, covariation=False, clock_rate=fixed_clock_rate)==ttconf.ERROR:
-                return ttconf.ERROR
+            self.reroot(root='least-squares' if reroot=='best' else reroot, covariation=False, clock_rate=fixed_clock_rate)
         else:
             self.get_clock_model(covariation=False, slope=fixed_clock_rate)
 
@@ -339,16 +336,23 @@ class TreeTime(ClockTree):
 
         residuals = np.array(list(res.values()))
         iqd = np.percentile(residuals,75) - np.percentile(residuals,25)
+        bad_branch_count = 0
         for node,r in res.items():
             if abs(r)>n_iqd*iqd and node.up.up is not None:
                 self.logger('TreeTime.ClockFilter: marking %s as outlier, residual %f interquartile distances'%(node.name,r/iqd), 3, warn=True)
                 node.bad_branch=True
+                bad_branch_count += 1
             else:
                 node.bad_branch=False
 
+        if bad_branch_count>0.34*self.tree.count_terminals():
+            self.logger("TreeTime.clock_filter: More than a third of leaves have been excluded by the clock filter. Please check your input data.", 0, warn=True)
+        # reassign bad_branch flags to internal nodes
+        self.prepare_tree()
+
         # redo root estimation after outlier removal
-        if reroot and self.reroot(root=reroot, clock_rate=fixed_clock_rate)==ttconf.ERROR:
-                return ttconf.ERROR
+        if reroot:
+            self.reroot(root=reroot, clock_rate=fixed_clock_rate)
 
         if plot:
             self.plot_root_to_tip()
@@ -414,6 +418,7 @@ class TreeTime(ClockTree):
 
         use_cov = self.use_covariation if covariation is None else covariation
         slope = 0.0 if type(root)==str and root.startswith('min_dev') else clock_rate
+        old_root = self.tree.root
 
         self.logger("TreeTime.reroot: with method or node: %s"%root,0)
         for n in self.tree.find_clades():
@@ -444,8 +449,7 @@ class TreeTime(ClockTree):
                                    if n.raw_date_constraint is not None],
                                    key=lambda x:np.mean(x.raw_date_constraint))[0]
             else:
-                self.logger('TreeTime.reroot -- ERROR: unsupported rooting mechanisms or root not found',0,warn=True)
-                return ttconf.ERROR
+                raise UnknownMethodError('TreeTime.reroot -- ERROR: unsupported rooting mechanisms or root not found')
 
             #this forces a bifurcating root, as we want. Branch lengths will be reoptimized anyway.
             #(Without outgroup_branch_length, gives a trifurcating root, but this will mean
@@ -454,9 +458,6 @@ class TreeTime(ClockTree):
             self.get_clock_model(covariation=use_cov, slope = slope)
 
 
-        if new_root == ttconf.ERROR:
-            return ttconf.ERROR
-
         self.logger("TreeTime.reroot: Tree was re-rooted to node "
                     +('new_node' if new_root.name is None else new_root.name), 2)
 
@@ -478,7 +479,7 @@ class TreeTime(ClockTree):
 
         self.get_clock_model(covariation=self.use_covariation, slope=slope)
 
-        return ttconf.SUCCESS
+        return new_root
 
 
     def resolve_polytomies(self, merge_compressed=False):
@@ -540,7 +541,7 @@ class TreeTime(ClockTree):
 
         from .branch_len_interpolator import BranchLenInterpolator
 
-        zero_branch_slope = self.gtr.mu*self.seq_len
+        zero_branch_slope = self.gtr.mu*self.data.full_length
 
         def _c_gain(t, n1, n2, parent):
             """
@@ -598,13 +599,13 @@ class TreeTime(ClockTree):
 
                 # set parameters for the new node
                 new_node.up = clade
+                new_node.tt = self
                 n1.up = new_node
                 n2.up = new_node
-                if hasattr(clade, "cseq"):
-                    new_node.cseq = clade.cseq
-                    self._store_compressed_sequence_to_node(new_node)
+                if hasattr(clade, "_cseq"):
+                    new_node._cseq = clade._cseq
+                    self.add_branch_state(new_node)
 
-                new_node.mutations = []
                 new_node.mutation_length = 0.0
                 new_node.branch_length_interpolator = BranchLenInterpolator(new_node, self.gtr, one_mutation=self.one_mutation,
                                                                             branch_length_mode = self.branch_length_mode)
@@ -860,10 +861,14 @@ def plot_vs_years(tt, step = None, ax=None, confidence=None, ticks=True, **kwarg
         tick_vals = [x+offset-shift for x in xticks]
 
     ax.set_xticks(xticks)
-    ax.set_xticklabels(map(str, tick_vals))
+    if step>=1:
+        tick_labels = ["%d"%(int(x)) for x in tick_vals]
+    else:
+        tick_labels = ["%1.2f"%(x) for x in tick_vals]
+    ax.set_xlim((0,date_range))
+    ax.set_xticklabels(tick_labels)
     ax.set_xlabel('year')
     ax.set_ylabel('')
-    ax.set_xlim((0,date_range))
 
     # put shaded boxes to delineate years
     if step:
@@ -878,7 +883,7 @@ def plot_vs_years(tt, step = None, ax=None, confidence=None, ticks=True, **kwarg
                           edgecolor=[1,1,1])
             ax.add_patch(r)
             if year in tick_vals and pos>=xlim[0] and pos<=xlim[1] and ticks:
-                label_str = str(step*(year//step)) if step<1 else  str(int(year))
+                label_str = "%1.2f"%(step*(year//step)) if step<1 else  str(int(year))
                 ax.text(pos,ylim[0]-0.04*(ylim[1]-ylim[0]), label_str,
                         horizontalalignment='center')
         ax.set_axis_off()
@@ -887,15 +892,13 @@ def plot_vs_years(tt, step = None, ax=None, confidence=None, ticks=True, **kwarg
     if confidence:
         tree_layout(tt.tree)
         if not hasattr(tt.tree.root, "marginal_inverse_cdf"):
-            print("marginal time tree reconstruction required for confidence intervals")
-            return ttconf.ERROR
+            raise NotReadyError("marginal time tree reconstruction required for confidence intervals")
         elif type(confidence) is float:
             cfunc = tt.get_max_posterior_region
         elif len(confidence)==2:
             cfunc = tt.get_confidence_interval
         else:
-            print("confidence needs to be either a float (for max posterior region) or a two numbers specifying lower and upper bounds")
-            return ttconf.ERROR
+            raise NotReadyError("confidence needs to be either a float (for max posterior region) or a two numbers specifying lower and upper bounds")
 
         for n in tt.tree.find_clades():
             pos = cfunc(n, confidence)
