@@ -7,7 +7,7 @@ from scipy.interpolate import interp1d
 from scipy.integrate import quad
 from scipy import stats
 from scipy.ndimage import binary_dilation
-from treetime import config as ttconf
+from treetime import TreeTimeError
 
 class DateConversion(object):
     """
@@ -101,7 +101,7 @@ class DateConversion(object):
 
     def to_numdate(self, tbp):
         """
-        Convert the numeric date to the branch-len scale
+        Convert time before present measured in clock rate units to numeric calendar dates
         """
         return numeric_date() - self.to_years(tbp)
 
@@ -150,19 +150,66 @@ def numeric_date(dt=None):
         date of to be converted. if None, assume today
 
     """
+    from calendar import isleap
+
     if dt is None:
         dt = datetime.datetime.now()
 
+    days_in_year = 366 if isleap(dt.year) else 365
     try:
-        res = dt.year + dt.timetuple().tm_yday / 365.25
+        res = dt.year + (dt.timetuple().tm_yday-0.5) / days_in_year
     except:
         res = None
 
     return res
 
 
+def datetime_from_numeric(numdate):
+    """convert a numeric decimal date to a python datetime object
+    Note that this only works for AD dates since the range of datetime objects
+    is restricted to year>1.
 
-def parse_dates(date_file):
+    Parameters
+    ----------
+    numdate : float
+        numeric date as in 2018.23
+
+    Returns
+    -------
+    datetime.datetime
+        datetime object
+    """
+    from calendar import isleap
+    days_in_year = 366 if isleap(int(numdate)) else 365
+    # add a small number of the time elapsed in a year to avoid
+    # unexpected behavior for values 1/365, 2/365, etc
+    days_elapsed = int(((numdate%1)+1e-10)*days_in_year)
+    date = datetime.datetime(int(numdate),1,1) + datetime.timedelta(days=days_elapsed)
+    return date
+
+
+def datestring_from_numeric(numdate):
+    """convert a numerical date to a formated date string YYYY-MM-DD
+
+    Parameters
+    ----------
+    numdate : float
+        numeric date as in 2018.23
+
+    Returns
+    -------
+    str
+        date string YYYY-MM-DD
+    """
+    if numdate>1900: # python datetime doesn't work for dates before 1900. This can be relaxed to numdate>1 once we drop python 2.7
+        return datetime.datetime.strftime(datetime_from_numeric(numdate), "%Y-%m-%d")
+    else:
+        year = int(np.floor(numdate))
+        dt = datetime_from_numeric(1900+(numdate%1))
+        return "%04d-%02d-%02d"%(year, dt.month, dt.day)
+
+
+def parse_dates(date_file, name_col=None, date_col=None):
     """
     parse dates from the arguments and return a dictionary mapping
     taxon names to numerical dates.
@@ -191,7 +238,7 @@ def parse_dates(date_file):
 
     try:
         # read the metadata file into pandas dataframe.
-        df = pd.read_csv(date_file, sep=full_sep, engine='python')
+        df = pd.read_csv(date_file, sep=full_sep, engine='python', dtype='str')
         # check the metadata has strain names in the first column
         # look for the column containing sampling dates
         # We assume that the dates might be given either in human-readable format
@@ -212,25 +259,37 @@ def parse_dates(date_file):
             if any([x==col.lower() for x in ['name', 'strain', 'accession']]):
                 potential_index_columns.append((ci, col))
 
+        if date_col and date_col not in df.columns:
+            raise TreeTimeError("ERROR: specified column for dates does not exist. \n\tAvailable columns are: "\
+                                +", ".join(df.columns)+"\n\tYou specified '%s'"%date_col)
+
+        if name_col and name_col not in df.columns:
+            raise TreeTimeError("ERROR: specified column for the taxon name does not exist. \n\tAvailable columns are: "\
+                                +", ".join(df.columns)+"\n\tYou specified '%s'"%name_col)
+
+
         dates = {}
         # if a potential numeric date column was found, use it
         # (use the first, if there are more than one)
-        if not len(potential_index_columns):
-            print("ERROR: Cannot read metadata: need at least one column that contains the taxon labels."
-                  " Looking for the first column that contains 'name', 'strain', or 'accession' in the header.", file=sys.stderr)
-            return dates
+        if not (len(potential_index_columns) or name_col):
+            raise TreeTimeError("ERROR: Cannot read metadata: need at least one column that contains the taxon labels."
+                  " Looking for the first column that contains 'name', 'strain', or 'accession' in the header.")
         else:
             # use the first column that is either 'name', 'strain', 'accession'
-            index_col = sorted(potential_index_columns)[0][1]
+            if name_col is None:
+                index_col = sorted(potential_index_columns)[0][1]
+            else:
+                index_col = name_col
             print("\tUsing column '%s' as name. This needs match the taxon names in the tree!!"%index_col)
 
-        if len(potential_date_columns)>=1:
+        if len(potential_date_columns)>=1 or date_col:
             #try to parse the csv file with dates in the idx column:
-            idx = potential_date_columns[0][0]
-            col_name = potential_date_columns[0][1]
-            print("\tUsing column '%s' as date."%col_name)
+            if date_col is None:
+                date_col = potential_date_columns[0][1]
+
+            print("\tUsing column '%s' as date."%date_col)
             for ri, row in df.iterrows():
-                date_str = row.loc[col_name]
+                date_str = row.loc[date_col]
                 k = row.loc[index_col]
                 # try parsing as a float first
                 try:
@@ -255,15 +314,16 @@ def parse_dates(date_file):
                             dates[k] = [numeric_date(x) for x in [lower, upper]]
 
         else:
-            print("ERROR: Metadata file has no column which looks like a sampling date!", file=sys.stderr)
+            raise TreeTimeError("ERROR: Metadata file has no column which looks like a sampling date!")
 
         if all(v is None for v in dates.values()):
-            print("ERROR: Cannot parse dates correctly! Check date format.", file=sys.stderr)
-            return {}
+            raise TreeTimeError("ERROR: Cannot parse dates correctly! Check date format.")
+        print(dates)
         return dates
+    except TreeTimeError as err:
+        raise err
     except:
-        print("ERROR: Cannot read the metadata file!", file=sys.stderr)
-        return {}
+        raise
 
 
 def ambiguous_date_to_date_range(mydate, fmt="%Y-%m-%d", min_max_year=None):
@@ -283,10 +343,9 @@ def ambiguous_date_to_date_range(mydate, fmt="%Y-%m-%d", min_max_year=None):
     tuple
         upper and lower bounds on the date. return (None, None) if errors
     """
-    from datetime import datetime
     sep = fmt.split('%')[1][-1]
     min_date, max_date = {}, {}
-    today = datetime.today().date()
+    today = datetime.date.today()
 
     for val, field  in zip(mydate.split(sep), fmt.split(sep+'%')):
         f = 'year' if 'y' in field.lower() else ('day' if 'd' in field.lower() else 'month')
@@ -315,8 +374,8 @@ def ambiguous_date_to_date_range(mydate, fmt="%Y-%m-%d", min_max_year=None):
                 return None, None
     max_date['day'] = min(max_date['day'], 31 if max_date['month'] in [1,3,5,7,8,10,12]
                                            else 28 if max_date['month']==2 else 30)
-    lower_bound = datetime(year=min_date['year'], month=min_date['month'], day=min_date['day']).date()
-    upper_bound = datetime(year=max_date['year'], month=max_date['month'], day=max_date['day']).date()
+    lower_bound = datetime.date(year=min_date['year'], month=min_date['month'], day=min_date['day'])
+    upper_bound = datetime.date(year=max_date['year'], month=max_date['month'], day=max_date['day'])
     return (lower_bound, upper_bound if upper_bound<today else today)
 
 
@@ -386,6 +445,7 @@ def build_newick_fasttree(aln_fname, nuc=True):
 
 def build_newick_raxml(aln_fname, nthreads=2, raxml_bin="raxml", **kwargs):
     import shutil,os
+    print("Building tree with raxml")
     from Bio import Phylo, AlignIO
     AlignIO.write(AlignIO.read(aln_fname, 'fasta'),"temp.phyx", "phylip-relaxed")
     cmd = raxml_bin + " -f d -T " + str(nthreads) + " -m GTRCAT -c 25 -p 235813 -n tre -s temp.phyx"
@@ -397,12 +457,33 @@ def build_newick_iqtree(aln_fname, nthreads=2, iqtree_bin="iqtree",
                         iqmodel="HKY",  **kwargs):
     import os
     from Bio import Phylo, AlignIO
-    with open(aln_fname) as ifile:
-        tmp_seqs = ifile.readlines()
+    print("Building tree with iqtree")
+    aln = None
+    for fmt in ['fasta', 'phylip-relaxed']:
+        try:
+            aln = AlignIO.read(aln_fname, fmt)
+            break
+        except:
+            continue
+
+    if aln is None:
+        raise ValueError("failed to read alignment for tree building")
+
     aln_file = "temp.fasta"
-    with open(aln_file, 'w') as ofile:
-        for line in tmp_seqs:
-            ofile.write(line.replace('/', '_X_X_').replace('|','_Y_Y_'))
+    seq_names = set()
+    for s in aln:
+        tmp  = s.id
+        for c, sub in zip('/|()', 'VWXY'):
+            tmp = tmp.replace(c, '_%s_%s_'%(sub,sub))
+        if tmp in seq_names:
+            print("A sequence with name {} already exists, skipping....".format(s.id))
+            continue
+        s.id = tmp
+        s.name = s.id
+        s.description = ''
+        seq_names.add(s.id)
+
+    AlignIO.write(aln, aln_file, 'fasta')
 
     fast_opts = [
         "-ninit", "2",
@@ -416,7 +497,10 @@ def build_newick_iqtree(aln_fname, nthreads=2, iqtree_bin="iqtree",
     os.system(" ".join(call))
     T = Phylo.read(aln_file+".treefile", 'newick')
     for n in T.get_terminals():
-        n.name = n.name.replace('_X_X_','/').replace('_Y_Y_','|')
+        tmp = n.name
+        for c, sub in zip('/|()', 'VWXY'):
+            tmp = tmp.replace('_%s_%s_'%(sub,sub), c)
+        n.name = tmp
     return T
 
 if __name__ == '__main__':
