@@ -1,9 +1,10 @@
 import numpy as np
 from collections import defaultdict
+from textwrap import fill
 
 ## Functions to read in and print out VCF files
 
-def read_vcf(vcf_file, ref_file):
+def read_vcf(vcf_file, ref_file=None):
     """
     Reads in a vcf/vcf.gz file and associated
     reference sequence fasta (to which the VCF file is mapped).
@@ -21,7 +22,7 @@ def read_vcf(vcf_file, ref_file):
     ----------
     vcf_file : string
         Path to the vcf or vcf.gz file to be read in
-    ref_file : string
+    ref_file : string, optional
         Path to the fasta reference file to be read in
 
     Returns
@@ -109,7 +110,7 @@ def read_vcf(vcf_file, ref_file):
             for i in range(len(ref)):
                 #if the pos doesn't match, store in sequences
                 if ref[i] != alt[i]:
-                    snps[pos+i] = alt[i] if alt[i] != '.' else 'N' #'.' = no-call
+                    snps[pos+i] = (alt[i] if alt[i] != '.' else 'N') #'.' = no-call
                 #if about to run out of ref, store rest:
                 if (i+1) >= len(ref):
                     ins[pos+i] = alt[i:]
@@ -122,7 +123,7 @@ def read_vcf(vcf_file, ref_file):
                 #if not, there may be mutations
                 else:
                     if ref[i] != alt[i]:
-                        snps[pos+i] = alt[i] if alt[i] != '.' else 'N' #'.' = no-call
+                        snps[pos+i] = (alt[i] if alt[i] != '.' else 'N') #'.' = no-call
         #Insertion
         elif len(alt) > 1:
             ins[pos] = alt
@@ -153,7 +154,7 @@ def read_vcf(vcf_file, ref_file):
                     #if not, there may be mutations
                     else:
                         if ref[i] != alt[i]:
-                            snps[pos+i] = alt[i] if alt[i] != '.' else 'N' #'.' = no-call
+                            snps[pos+i] = (alt[i] if alt[i] != '.' else 'N') #'.' = no-call
 
         #If not deletion, need to know call type
         #if het, see if proposed alt is 1bp mutation
@@ -194,8 +195,7 @@ def read_vcf(vcf_file, ref_file):
         for line in f:
             if line[0] != '#':
                 #actual data - most common so first in 'if-list'!
-                line = line.strip()
-                dat = line.split('\t')
+                dat = line.strip().split('\t')
                 POS = int(dat[posLoc])
                 REF = dat[refLoc]
                 ALT = dat[altLoc].split(',')
@@ -257,9 +257,12 @@ def read_vcf(vcf_file, ref_file):
         for s in missings:
             sequences[s] = {}
 
-    refSeq = SeqIO.read(ref_file, format='fasta')
-    refSeq = refSeq.upper() #convert to uppercase to avoid unknown chars later
-    refSeqStr = str(refSeq.seq)
+    if ref_file:
+        refSeq = SeqIO.read(ref_file, format='fasta')
+        refSeq = refSeq.upper() #convert to uppercase to avoid unknown chars later
+        refSeqStr = str(refSeq.seq)
+    else:
+        refSeqStr = None
 
     compress_seq = {'reference':refSeqStr,
                     'sequences': sequences,
@@ -335,7 +338,7 @@ def write_vcf(tree_dict, file_name):#, compress=False):
                     pattern.append(sequences[k][pi])
                 except KeyError:
                     pattern.append(ref[pi])
-            pattern = np.array(pattern)
+            pattern = np.array(pattern).astype('U')
 
         sites = []
         sites.append(pattern)
@@ -350,7 +353,7 @@ def write_vcf(tree_dict, file_name):#, compress=False):
                     pattern.append(sequences[k][pi])
                 except KeyError:
                     pattern.append(ref[pi])
-            pattern = np.array(pattern)
+            pattern = np.array(pattern).astype('U')
 
             #Stops 'greedy' behaviour from adding mutations adjacent to deletions
             if any(pattern == '-'): #if part of deletion, append
@@ -360,9 +363,7 @@ def write_vcf(tree_dict, file_name):#, compress=False):
                 i-=1    #don't append, break this loop
 
         #Rotate them into 'calls'
-        sites = np.asarray(sites)
-        align = np.rot90(sites)
-        align = np.flipud(align)
+        align = np.asarray(sites).T
 
         #Get rid of '-', and put '.' for calls that match ref
         #Only removes trailing '-'. This breaks VCF convension, but the standard
@@ -387,11 +388,14 @@ def write_vcf(tree_dict, file_name):#, compress=False):
 
     #prepare the header of the VCF & write out
     header=["#CHROM","POS","ID","REF","ALT","QUAL","FILTER","INFO","FORMAT"]+list(sequences.keys())
-    with open(file_name, 'w') as the_file:
-        the_file.write( "##fileformat=VCFv4.2\n"+
+
+    opn = gzip.open if file_name.endswith(('.gz', '.GZ')) else open
+    out_file = opn(file_name, 'w')
+
+    out_file.write( "##fileformat=VCFv4.2\n"+
                         "##source=NextStrain\n"+
                         "##FORMAT=<ID=GT,Number=1,Type=String,Description=\"Genotype\">\n")
-        the_file.write("\t".join(header)+"\n")
+    out_file.write("\t".join(header)+"\n")
 
     vcfWrite = []
     errorPositions = []
@@ -427,8 +431,8 @@ def write_vcf(tree_dict, file_name):#, compress=False):
             except KeyError:
                 pattern2.append(ref[pi+1])
 
-        pattern = np.array(pattern)
-        pattern2 = np.array(pattern2)
+        pattern = np.array(pattern).astype('U')
+        pattern2 = np.array(pattern2).astype('U')
 
         #If a deletion here, need to gather up all bases, and position before
         if any(pattern == '-'):
@@ -439,7 +443,9 @@ def write_vcf(tree_dict, file_name):#, compress=False):
                 #If theres a deletion in 1st pos, VCF files do not handle this well.
                 #Proceed keeping it as '-' for alt (violates VCF), but warn user to check output.
                 #(This is rare)
-                print ("WARNING: You have a deletion in the first position of your alignment. VCF format does not handle this well. Please check the output to ensure it is correct.")
+                print(fill("WARNING: You have a deletion in the first position of your"
+                           " alignment. VCF format does not handle this well. Please check"
+                           " the output to ensure it is correct."))
         else:
             #If a deletion in next pos, need to gather up all bases
             if any(pattern2 == '-'):
@@ -493,26 +499,119 @@ def write_vcf(tree_dict, file_name):#, compress=False):
     #won't be counted in the below list, which is only sites removed from the VCF.
 
     if 'inferred_const_sites' in tree_dict and explainedErrors != 0:
-        print ( "Sites that were constant except for ambiguous bases were made constant by TreeTime. This happened {} times. These sites are now excluded from the VCF.".format(explainedErrors))
+        print(fill("Sites that were constant except for ambiguous bases were made" +
+                   " constant by TreeTime. This happened {} times. These sites are".format(explainedErrors) +
+                   " now excluded from the VCF."))
 
     if len(errorPositions) != 0:
-        print ("\n***WARNING: vcf_utils.py"
-            "\n{} sites were found that had no alternative bases. If this data has been "
-            "run through TreeTime and contains ambiguous bases, try calling get_tree_dict with "
-            "var_ambigs=True to see if this clears the error."
-            "\n\nAlternative causes:"
-            "\n- Not all sequences in your alignment are in the tree (if you are running TreeTime via commandline "
-            "this is most likely)"
-            "\n- In TreeTime, can be caused by overwriting variants in tips with small branch lengths (debug)"
-            "\n\nThese are the positions affected (numbering starts at 0):".format(str(len(errorPositions))))
-        print (",".join(errorPositions))
+        print ("\n***WARNING: vcf_utils.py")
+        print(fill("\n{} sites were found that had no alternative bases.".format(str(len(errorPositions)))+
+                  " If this data has been run through TreeTime and contains ambiguous bases,"
+                  " try calling get_tree_dict with var_ambigs=True to see if this clears the error."))
+        print(fill("\nAlternative causes:"
+                   "\n- Not all sequences in your alignment are in the tree"
+                  " (if you are running TreeTime via commandline this is most likely)"
+                  "\n- In TreeTime, can be caused by overwriting variants in tips with small branch lengths (debug)"
+                  "\n\nThese are the positions affected (numbering starts at 0):"))
+        print(fill(", ".join(errorPositions)))
+
+    out_file.write("\n".join(vcfWrite))
+    out_file.close()
+
+
+def process_sparse_alignment(aln, ref, ambiguous_char):
+    return process_alignment_dictionary(aln, ref, ambiguous_char)
+
+def process_alignment_dictionary(aln, ref, ambiguous_char):
+    """
+    prepare the dictionary specifying differences from a reference sequence
+    to construct the reduced alignment with variable sites only. NOTE:
+        - sites can be constant but different from the reference
+        - sites can be constant plus a ambiguous sites
+
+    assigns
+    -------
+    - self.nonref_positions: at least one sequence is different from ref
+
+    Returns
+    -------
+    reduced_alignment_const
+        reduced alignment accounting for non-variable postitions
+
+    alignment_patterns_const
+        dict pattern -> (pos in reduced alignment, list of pos in full alignment)
+
+    variable_positions
+        list of variable positions needed to construct remaining
+
+    """
+
+    # number of sequences in alignment
+    nseq = len(aln)
+
+    inv_map = defaultdict(list)
+    for k,v in aln.items():
+        for pos, bs in v.items():
+            inv_map[pos].append(bs)
+
+    nonref_positions = np.sort(list(inv_map.keys()))
+    constant_up_to_ambiguous = []
+
+    nonref_const = []
+    nonref_alleles = []
+    ambiguous_const = []
+    variable_pos = []
+    for pos, bs in inv_map.items(): #loop over positions and patterns
+        bases = list(np.unique(bs))
+        if len(bs) == nseq: #every sequence is different from reference
+            if (len(bases)<=2 and ambiguous_char in bases) or len(bases)==1:
+                # all sequences different from reference, but only one state
+                # (other than ambiguous_char) in column
+                nonref_const.append(pos)
+                if len(bases)==1:
+                    nonref_alleles.append(bases[0])
+                else:
+                    nonref_alleles.append([x for x in bases if x!=ambiguous_char][0])
+
+                if ambiguous_char in bases: #keep track of sites 'made constant'
+                    constant_up_to_ambiguous.append(pos)
+            else:
+                # at least two non-reference alleles
+                variable_pos.append(pos)
+        else: # not every sequence different from reference
+            if len(bases)==1 and bases[0]==ambiguous_char:
+                ambiguous_const.append(pos)
+                constant_up_to_ambiguous.append(pos) #keep track of sites 'made constant'
+            else:
+                # at least one non ambiguous non-reference allele not in
+                # every sequence
+                variable_pos.append(pos)
+
+    refMod = np.copy(ref)
+    # place constant non reference positions by their respective allele
+    refMod[nonref_const] = nonref_alleles
+    # mask variable positions
+    states = np.unique(refMod)
+    refMod[variable_pos] = '.'
+
+    # for each base in the gtr, make constant alignment pattern and
+    # assign it to all const positions in the modified reference sequence
+    constant_columns = []
+    constant_patterns = {}
+    for base in states:
+        if base==ambiguous_char:
+            continue
+        p = np.repeat(base, nseq)
+        pos = list(np.where(refMod==base)[0])
+        #if the alignment doesn't have a const site of this base, don't add! (ex: no '----' site!)
+        if len(pos):
+            constant_patterns["".join(p.astype('U'))] = [len(constant_columns), pos]
+            constant_columns.append(p)
+
+    return {"constant_columns": constant_columns,
+            "constant_patterns": constant_patterns,
+            "variable_positions": variable_pos,
+            "nonref_positions": nonref_positions,
+            "constant_up_to_ambiguous": constant_up_to_ambiguous}
 
-    with open(file_name, 'a') as the_file:
-        the_file.write("\n".join(vcfWrite))
 
-    if file_name.endswith(('.gz', '.GZ')):
-        import os
-        #must temporarily remove .gz ending, or gzip won't zip it!
-        os.rename(file_name, file_name[:-3])
-        call = ["gzip", file_name[:-3]]
-        os.system(" ".join(call))
