@@ -2,6 +2,7 @@ from __future__ import print_function, division, absolute_import
 import os, shutil, sys
 import numpy as np
 import pandas as pd
+from textwrap import fill
 from Bio.SeqRecord import SeqRecord
 from Bio.Seq import Seq
 from Bio.Align import MultipleSeqAlignment
@@ -10,8 +11,9 @@ from Bio import __version__ as bioversion
 from treetime import TreeAnc, GTR, TreeTime
 from treetime import config as ttconf
 from treetime import utils
-from treetime.vcf_utils import read_vcf, write_vcf
-from treetime.seq_utils import alphabets
+from .vcf_utils import read_vcf, write_vcf
+from .seq_utils import alphabets
+from treetime import TreeTimeError
 
 def assure_tree(params, tmp_dir='treetime_tmp'):
     """
@@ -59,10 +61,10 @@ def create_gtr(params):
 
             gtr = GTR.standard(model, **kwargs)
             infer_gtr = False
-        except:
-            print ("Could not create GTR model from input arguments. Using default (Jukes-Cantor 1969)")
-            gtr = GTR.standard('jc', alphabet='aa' if params.aa else 'nuc')
-            infer_gtr = False
+        except KeyError as e:
+            print("\nUNKNOWN SUBSTITUTION MODEL\n")
+            raise e
+
     return gtr
 
 def get_outdir(params, suffix='_treetime'):
@@ -77,9 +79,14 @@ def get_outdir(params, suffix='_treetime'):
             return params.outdir.rstrip('/') + '/'
 
     from datetime import datetime
-    outdir = datetime.now().date().isoformat()+suffix.rstrip('/') + '/'
-    if not os.path.exists(outdir):
-        os.makedirs(outdir)
+    outdir_stem = datetime.now().date().isoformat()
+    outdir = outdir_stem + suffix.rstrip('/')+'/'
+    count = 1
+    while os.path.exists(outdir):
+        outdir = outdir_stem + '-%04d'%count + suffix.rstrip('/')+'/'
+        count += 1
+
+    os.makedirs(outdir)
     return outdir
 
 def get_basename(params, outdir):
@@ -157,15 +164,15 @@ def plot_rtt(tt, fname):
 
 
 def export_sequences_and_tree(tt, basename, is_vcf=False, zero_based=False,
-                              report_ambiguous=False, timetree=False, confidence=False):
+                              report_ambiguous=False, timetree=False, confidence=False,
+                              reconstruct_tip_states=False):
     seq_info = is_vcf or tt.aln
     if is_vcf:
-        tt.recover_var_ambigs()
         outaln_name = basename + 'ancestral_sequences.vcf'
-        write_vcf(tt.get_tree_dict(keep_var_ambigs=True), outaln_name)
+        write_vcf(tt.get_reconstructed_alignment(reconstruct_tip_states=reconstruct_tip_states), outaln_name)
     elif tt.aln:
         outaln_name = basename + 'ancestral_sequences.fasta'
-        AlignIO.write(tt.get_reconstructed_alignment(), outaln_name, 'fasta')
+        AlignIO.write(tt.get_reconstructed_alignment(reconstruct_tip_states=reconstruct_tip_states), outaln_name, 'fasta')
     if seq_info:
         print("\n--- alignment including ancestral nodes saved as  \n\t %s\n"%outaln_name)
 
@@ -183,10 +190,16 @@ def export_sequences_and_tree(tt, basename, is_vcf=False, zero_based=False,
     for n in tt.tree.find_clades():
         if timetree:
             if confidence:
-                conf = tt.get_max_posterior_region(n, fraction=0.9)
-                fh_dates.write('%s\t%s\t%f\t%f\t%f\n'%(n.name, n.date, n.numdate,conf[0], conf[1]))
+                if n.bad_branch:
+                    fh_dates.write('%s\t--\t--\t--\t--\n'%(n.name))
+                else:
+                    conf = tt.get_max_posterior_region(n, fraction=0.9)
+                    fh_dates.write('%s\t%s\t%f\t%f\t%f\n'%(n.name, n.date, n.numdate,conf[0], conf[1]))
             else:
-                fh_dates.write('%s\t%s\t%f\n'%(n.name, n.date, n.numdate))
+                if n.bad_branch:
+                    fh_dates.write('%s\t--\t--\n'%(n.name))
+                else:
+                    fh_dates.write('%s\t%s\t%f\n'%(n.name, n.date, n.numdate))
 
         n.confidence=None
         # due to a bug in older versions of biopython that truncated filenames in nexus export
@@ -205,7 +218,7 @@ def export_sequences_and_tree(tt, basename, is_vcf=False, zero_based=False,
             n.comment+=(',' if n.comment else '&') + 'date=%1.2f'%n.numdate
 
     # write tree to file
-    fmt_bl = "%1.6f" if tt.seq_len<1e6 else "%1.8e"
+    fmt_bl = "%1.6f" if tt.data.full_length<1e6 else "%1.8e"
     if timetree:
         outtree_name = basename + 'timetree.nexus'
         print("--- saved divergence times in \n\t %s\n"%dates_fname)
@@ -278,7 +291,7 @@ def scan_homoplasies(params):
     if is_vcf:
         L = len(ref) + params.const
     else:
-        L = treeanc.aln.get_alignment_length() + params.const
+        L = treeanc.data.full_length + params.const
 
     N_seq = len(treeanc.aln)
     N_tree = treeanc.tree.count_terminals()
@@ -294,11 +307,6 @@ def scan_homoplasies(params):
     ndiff = treeanc.infer_ancestral_sequences('ml', infer_gtr=params.gtr=='infer',
                                       marginal=False, fixed_pi=fixed_pi)
     print("...done.")
-    if ndiff==ttconf.ERROR: # if reconstruction failed, exit
-        print("Something went wrong during ancestral reconstruction, please check your input files.", file=sys.stderr)
-        return 1
-    else:
-        print("...done.")
 
     if is_vcf:
         treeanc.recover_var_ambigs()
@@ -470,7 +478,7 @@ def timetree(params):
         relaxed_clock_params={'slack':params.relax[0], 'coupling':params.relax[1]}
 
 
-    dates = utils.parse_dates(params.dates)
+    dates = utils.parse_dates(params.dates, date_col=params.date_column, name_col=params.name_column)
     if len(dates)==0:
         print("No valid dates -- exiting.")
         return 1
@@ -506,7 +514,7 @@ def timetree(params):
         return 1
     myTree = TreeTime(dates=dates, tree=params.tree, ref=ref,
                       aln=aln, gtr=gtr, seq_len=params.sequence_length,
-                      verbose=params.verbose)
+                      verbose=params.verbose, fill_overhangs=not params.keep_overhangs)
     myTree.tip_slack=params.tip_slack
     if not myTree.one_mutation:
         print("TreeTime setup failed, exiting")
@@ -533,9 +541,9 @@ def timetree(params):
     elif params.confidence and params.covariation:
         vary_rate = True
     elif params.confidence:
-        print("\nOutside of covariation aware mode TreeTime cannot estimate confidence intervals "
-                "without specified standard deviation of the clock rate. \nPlease specify '--clock-std-dev' "
-                "or rerun with '--covariation'. \nWill proceed without confidence estimation")
+        print(fill("Outside of covariation aware mode TreeTime cannot estimate confidence intervals "
+                "without specified standard deviation of the clock rate.Please specify '--clock-std-dev' "
+                "or rerun with '--covariation'. Will proceed without confidence estimation"))
         vary_rate = False
         calc_confidence = False
     else:
@@ -543,7 +551,8 @@ def timetree(params):
 
     # RUN
     root = None if params.keep_root else params.reroot
-    success = myTree.run(root=root, relaxed_clock=relaxed_clock_params,
+    try:
+        success = myTree.run(root=root, relaxed_clock=relaxed_clock_params,
                resolve_polytomies=(not params.keep_polytomies),
                Tc=coalescent, max_iter=params.max_iter,
                fixed_clock_rate=params.clock_rate,
@@ -551,11 +560,12 @@ def timetree(params):
                time_marginal="assign" if calc_confidence else False,
                vary_rate = vary_rate,
                branch_length_mode = branch_length_mode,
+               reconstruct_tip_states=params.reconstruct_tip_states,
                fixed_pi=fixed_pi,
-               use_covariation = params.covariation)
-    if success==ttconf.ERROR: # if TreeTime.run failed, exit
+               use_covariation = params.covariation, n_points=params.n_skyline)
+    except TreeTimeError as e:
         print("\nTreeTime run FAILED: please check above for errors and/or rerun with --verbose 4.\n")
-        return 1
+        raise e
 
     ###########################################################################
     ### OUTPUT and saving of results
@@ -593,7 +603,7 @@ def timetree(params):
                                       or params.tip_labels) else '')
 
     plot_vs_years(myTree, show_confidence=False, label_func=label_func,
-                  confidence=0.9 if params.confidence else None)
+                  confidence=0.9 if calc_confidence else None)
     tree_fname = (outdir + params.plot_tree)
     plt.savefig(tree_fname)
     print("--- saved tree as \n\t %s\n"%tree_fname)
@@ -611,7 +621,8 @@ def timetree(params):
                 fh.write("%s\t%1.3e\t%1.3e\t%1.3e\t%1.2f\n"%(n.name, n.clock_length, n.mutation_length, myTree.date2dist.clock_rate*g, g))
 
     export_sequences_and_tree(myTree, basename, is_vcf, params.zero_based,
-                              timetree=True, confidence=calc_confidence)
+                              timetree=True, confidence=calc_confidence,
+                              reconstruct_tip_states=params.reconstruct_tip_states)
 
     return 0
 
@@ -639,10 +650,14 @@ def ancestral_reconstruction(params):
 
     treeanc = TreeAnc(params.tree, aln=aln, ref=ref, gtr=gtr, verbose=1,
                       fill_overhangs=not params.keep_overhangs)
-    ndiff =treeanc.infer_ancestral_sequences('ml', infer_gtr=params.gtr=='infer',
-                                             marginal=params.marginal, fixed_pi=fixed_pi)
-    if ndiff==ttconf.ERROR: # if reconstruction failed, exit
-        return 1
+
+    try:
+        ndiff = treeanc.infer_ancestral_sequences('ml', infer_gtr=params.gtr=='infer',
+                                             marginal=params.marginal, fixed_pi=fixed_pi,
+                                             reconstruct_tip_states=params.reconstruct_tip_states)
+    except TreeTimeError as e:
+        print("\nAncestral reconstruction failed, please see above for error messages and/or rerun with --verbose 4\n")
+        raise e
 
     ###########################################################################
     ### OUTPUT and saving of results
@@ -655,20 +670,55 @@ def ancestral_reconstruction(params):
         print(treeanc.gtr)
 
     export_sequences_and_tree(treeanc, basename, is_vcf, params.zero_based,
-                              report_ambiguous=params.report_ambiguous)
+                              report_ambiguous=params.report_ambiguous,
+                              reconstruct_tip_states=params.reconstruct_tip_states)
 
     return 0
 
 def reconstruct_discrete_traits(tree, traits, missing_data='?', pc=1.0, sampling_bias_correction=None,
-                                weights=None, verbose=0):
+                                weights=None, verbose=0, iterations=5):
+    """take a set of discrete states associated with tips of a tree
+    and reconstruct their ancestral states along with a GTR model that
+    approximately maximizes the likelihood of the states on the tree.
+
+    Parameters
+    ----------
+    tree : str, Bio.Phylo.Tree
+        name of tree file or Biopython tree object
+    traits : dict
+        dictionary linking tips to straits
+    missing_data : str, optional
+        string indicating missing data
+    pc : float, optional
+        number of pseudo-counts to be used during GTR inference, default 1.0
+    sampling_bias_correction : float, optional
+        factor to inflate overall switching rate by to counteract sampling bias
+    weights : str, optional
+        name of file with equilibirum frequencies
+    verbose : int, optional
+        level of verbosity in output
+    iterations : int, optional
+        number of times non-linear optimization of overall rate and
+        transmission estimation are iterated
+
+    Returns
+    -------
+    tuple
+        tuple of treeanc object, forward and reverse alphabets
+
+    Raises
+    ------
+    TreeTimeError
+        raise error if ancestral reconstruction errors out
+    """
     unique_states = sorted(set(traits.values()))
     nc = len(unique_states)
     if nc>180:
         print("mugration: can't have more than 180 states!", file=sys.stderr)
-        return 1
+        return None, None, None
     elif nc<2:
         print("mugration: only one or zero states found -- this doesn't make any sense", file=sys.stderr)
-        return 1
+        return None, None, None
 
     ###########################################################################
     ### make a single character alphabet that maps to discrete states
@@ -699,6 +749,7 @@ def reconstruct_discrete_traits(tree, traits, missing_data='?', pc=1.0, sampling
     mugration_GTR.profile_map[missing_char] = np.ones(nc)
     mugration_GTR.ambiguous=missing_char
 
+
     ###########################################################################
     ### set up treeanc
     ###########################################################################
@@ -709,19 +760,35 @@ def reconstruct_discrete_traits(tree, traits, missing_data='?', pc=1.0, sampling
                    seq=Seq(reverse_alphabet[traits[n.name]]
                            if n.name in traits else missing_char))
                    for n in treeanc.tree.get_terminals()]
+    valid_seq = np.array([str(s.seq)!=missing_char for s in pseudo_seqs])
+    print("Assigned discrete traits to %d out of %d taxa.\n"%(np.sum(valid_seq),len(valid_seq)))
     treeanc.aln = MultipleSeqAlignment(pseudo_seqs)
 
-    ndiff = treeanc.infer_ancestral_sequences(method='ml', infer_gtr=True,
+    try:
+        ndiff = treeanc.infer_ancestral_sequences(method='ml', infer_gtr=True,
             store_compressed=False, pc=pc, marginal=True, normalized_rate=False,
-            fixed_pi=weights)
+            fixed_pi=weights, reconstruct_tip_states=True)
+        treeanc.optimize_gtr_rate()
+    except TreeTimeError as e:
+        print("\nAncestral reconstruction failed, please see above for error messages and/or rerun with --verbose 4\n")
+        raise e
 
-    if ndiff==ttconf.ERROR: # if reconstruction failed, exit
-        return 1
+    for i in range(iterations):
+        treeanc.infer_gtr(marginal=True, normalized_rate=False, pc=pc, fixed_pi=weights)
+        treeanc.optimize_gtr_rate()
 
     if sampling_bias_correction:
         treeanc.gtr.mu *= sampling_bias_correction
-        treeanc.infer_ancestral_sequences(infer_gtr=False, store_compressed=False,
-                                     marginal=True, normalized_rate=False)
+
+    treeanc.infer_ancestral_sequences(infer_gtr=False, store_compressed=False,
+                                 marginal=True, normalized_rate=False,
+                                 reconstruct_tip_states=True)
+
+    print(fill("NOTE: previous versions (<0.7.0) of this command made a 'short-branch length assumption. "
+          "TreeTime now optimizes the overall rate numerically and thus allows for long branches "
+          "along which multiple changes accumulated. This is expected to affect estimates of the "
+          "overall rate while leaving the relative rates mostly unchanged."))
+
     return treeanc, letter_to_state, reverse_alphabet
 
 
@@ -742,7 +809,19 @@ def mugration(params):
 
     outdir = get_outdir(params, '_mugration')
 
-    taxon_name = 'name' if 'name' in states.columns else states.columns[0]
+    if params.name_column:
+        if params.name_column in states.columns:
+            taxon_name = params.name_column
+        else:
+            print("Error: specified column '%s' for taxon name not found in meta data file with columns: "%params.name_column + " ".join(states.columns))
+            return 1
+    elif 'name' in states.columns: taxon_name = 'name'
+    elif 'strain' in states.columns: taxon_name = 'strain'
+    elif 'accession' in states.columns: taxon_name = 'accession'
+    else:
+        taxon_name = states.columns[0]
+    print("Using column '%s' as taxon name. This needs to match the taxa in the tree!"%taxon_name)
+
     if params.attribute:
         if params.attribute in states.columns:
             attr = params.attribute
@@ -754,11 +833,15 @@ def mugration(params):
         attr = states.columns[1]
         print("Attribute for mugration inference was not specified. Using "+attr, file=sys.stderr)
 
-    leaf_to_attr = {x[taxon_name]:x[attr] for xi, x in states.iterrows()
-                    if x[attr]!=params.missing_data}
+    leaf_to_attr = {x[taxon_name]:str(x[attr]) for xi, x in states.iterrows()
+                    if x[attr]!=params.missing_data and x[attr]}
 
     mug, letter_to_state, reverse_alphabet = reconstruct_discrete_traits(params.tree, leaf_to_attr, missing_data=params.missing_data,
-            pc=params.pc, sampling_bias_correction=params.sampling_bias_correction, verbose=params.verbose)
+            pc=params.pc, sampling_bias_correction=params.sampling_bias_correction, verbose=params.verbose, weights=params.weights)
+
+    if mug is None:
+        print("Mugration inference failed, check error messages above and your input data.")
+        return 1
 
     unique_states = sorted(letter_to_state.values())
     ###########################################################################
@@ -777,20 +860,20 @@ def mugration(params):
 
     terminal_count = 0
     for n in mug.tree.find_clades():
+        n.confidence=None
         if n.up is None:
             continue
-        n.confidence=None
         # due to a bug in older versions of biopython that truncated filenames in nexus export
         # we truncate them by hand and make them unique.
         if n.is_terminal() and len(n.name)>40 and bioversion<"1.69":
             n.name = n.name[:35]+'_%03d'%terminal_count
             terminal_count+=1
-        n.comment= '&%s="'%attr + letter_to_state[n.sequence[0]] +'"'
+        n.comment= '&%s="'%attr + letter_to_state[n.cseq[0]] +'"'
 
     if params.confidence:
         conf_name = basename+'confidence.csv'
         with open(conf_name, 'w') as ofile:
-            ofile.write('#name, '+', '.join(unique_states)+'\n')
+            ofile.write('#name, '+', '.join(mug.gtr.alphabet)+'\n')
             for n in mug.tree.find_clades():
                 ofile.write(n.name + ', '+', '.join([str(x) for x in n.marginal_profile[0]])+'\n')
         print("Saved table with ancestral state confidences as:", conf_name)
@@ -799,6 +882,7 @@ def mugration(params):
     outtree_name = basename+'annotated_tree.nexus'
     Phylo.write(mug.tree, outtree_name, 'nexus')
     print("Saved annotated tree as:", outtree_name)
+    print("---Done!\n")
 
     return 0
 
@@ -810,7 +894,7 @@ def estimate_clock_model(params):
 
     if assure_tree(params, tmp_dir='clock_model_tmp'):
         return 1
-    dates = utils.parse_dates(params.dates)
+    dates = utils.parse_dates(params.dates, date_col=params.date_column, name_col=params.name_column)
     if len(dates)==0:
         return 1
 
@@ -831,14 +915,15 @@ def estimate_clock_model(params):
         return 1
 
     basename = get_basename(params, outdir)
-    myTree = TreeTime(dates=dates, tree=params.tree, aln=aln, gtr='JC69',
+    try:
+        myTree = TreeTime(dates=dates, tree=params.tree, aln=aln, gtr='JC69',
                       verbose=params.verbose, seq_len=params.sequence_length,
                       ref=ref)
-    myTree.tip_slack=params.tip_slack
-    if myTree.tree is None:
-        print("ERROR: tree loading failed. exiting...")
-        return 1
+    except TreeTimeError as e:
+        print("\nTreeTime setup failed. Please see above for error messages and/or rerun with --verbose 4\n")
+        raise e
 
+    myTree.tip_slack=params.tip_slack
     if params.clock_filter:
         n_bad = [n.name for n in myTree.tree.get_terminals() if n.bad_branch]
         myTree.clock_filter(n_iqd=params.clock_filter, reroot=params.reroot or 'least-squares')
@@ -854,27 +939,27 @@ def estimate_clock_model(params):
             myTree.run(root="least-squares", max_iter=0,
                        use_covariation=params.covariation)
 
-        res = myTree.reroot(params.reroot,
+        try:
+            res = myTree.reroot(params.reroot,
                       force_positive=not params.allow_negative_rate)
-        myTree.get_clock_model(covariation=params.covariation)
+        except TreeTimeError as e:
+            print("ERROR: unknown root or rooting mechanism!")
+            raise e
 
-        if res==ttconf.ERROR:
-            print("ERROR: unknown root or rooting mechanism!\n"
-                  "\tvalid choices are 'least-squares', 'ML', and 'ML-rough'")
-            return 1
+        myTree.get_clock_model(covariation=params.covariation)
     else:
         myTree.get_clock_model(covariation=params.covariation)
 
     d2d = utils.DateConversion.from_regression(myTree.clock_model)
     print('\n',d2d)
-    print('The R^2 value indicates the fraction of variation in'
-          '\nroot-to-tip distance explained by the sampling times.'
-          '\nHigher values corresponds more clock-like behavior (max 1.0).')
+    print(fill('The R^2 value indicates the fraction of variation in'
+          'root-to-tip distance explained by the sampling times.'
+          'Higher values corresponds more clock-like behavior (max 1.0).')+'\n')
 
-    print('\nThe rate is the slope of the best fit of the date to'
-          '\nthe root-to-tip distance and provides an estimate of'
-          '\nthe substitution rate. The rate needs to be positive!'
-          '\nNegative rates suggest an inappropriate root.\n')
+    print(fill('The rate is the slope of the best fit of the date to'
+          'the root-to-tip distance and provides an estimate of'
+          'the substitution rate. The rate needs to be positive!'
+          'Negative rates suggest an inappropriate root.')+'\n')
 
     print('\nThe estimated rate and tree correspond to a root date:')
     if params.covariation:
